@@ -1,16 +1,21 @@
-mod alacritty_functions;
+pub mod alacritty_functions;
 mod floating_panes;
 pub mod grid;
 pub mod link_handler;
 mod plugin_pane;
+mod search;
 pub mod selection;
+pub mod sixel;
 pub mod terminal_character;
 mod terminal_pane;
+mod tiled_panes;
 
 pub use alacritty_functions::*;
 pub use floating_panes::*;
 pub use grid::*;
 pub use link_handler::*;
 pub(crate) use plugin_pane::*;
+pub use sixel::*;
 pub(crate) use terminal_character::*;
 pub use terminal_pane::*;
+pub use tiled_panes::*;
