@@ -2,9 +2,9 @@
 
 use super::command::RunCommandAction;
 use super::layout::TabLayout;
+use crate::data::InputMode;
 use crate::input::options::OnForceClose;
 use serde::{Deserialize, Serialize};
-use zellij_tile::data::InputMode;
 
 use crate::position::Position;
 
@@ -17,7 +17,7 @@ pub enum Direction {
     Down,
 }
 
-#[derive(Clone, Debug, PartialEq, Deserialize, Serialize)]
+#[derive(Clone, Debug, PartialEq, Eq, Deserialize, Serialize)]
 pub enum ResizeDirection {
     Left,
     Right,
@@ -27,12 +27,25 @@ pub enum ResizeDirection {
     Decrease,
 }
 
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Deserialize, Serialize)]
+pub enum SearchDirection {
+    Down,
+    Up,
+}
+
+#[derive(Clone, Debug, PartialEq, Eq, Deserialize, Serialize)]
+pub enum SearchOption {
+    CaseSensitivity,
+    WholeWord,
+    Wrap,
+}
+
 // As these actions are bound to the default config, please
 // do take care when refactoring - or renaming.
 // They might need to be adjusted in the default config
 // as well `../../assets/config/default.yaml`
 /// Actions that can be bound to keys.
-#[derive(Clone, Debug, PartialEq, Deserialize, Serialize)]
+#[derive(Clone, Debug, PartialEq, Eq, Deserialize, Serialize)]
 pub enum Action {
     /// Quit Zellij.
     Quit,
@@ -91,6 +104,7 @@ pub enum Action {
     /// Close the focus pane.
     CloseFocus,
     PaneNameInput(Vec<u8>),
+    UndoRenamePane,
     /// Create a new tab, optionally with a specified tab layout.
     NewTab(Option<TabLayout>),
     /// Do nothing.
@@ -104,7 +118,8 @@ pub enum Action {
     GoToTab(u32),
     ToggleTab,
     TabNameInput(Vec<u8>),
-    /// Run speficied command in new pane.
+    UndoRenameTab,
+    /// Run specified command in new pane.
     Run(RunCommandAction),
     /// Detach session and exit
     Detach,
@@ -119,6 +134,12 @@ pub enum Action {
     Deny,
     /// Confirm an action that invokes a prompt automatically
     SkipConfirm(Box<Action>),
+    /// Search for String
+    SearchInput(Vec<u8>),
+    /// Search for something
+    Search(SearchDirection),
+    /// Toggle case sensitivity of search
+    SearchToggleOption(SearchOption),
 }
 
 impl From<OnForceClose> for Action {
@@ -129,3 +150,13 @@ impl From<OnForceClose> for Action {
         }
     }
 }
+
+#[derive(Clone, Debug, PartialEq, Deserialize, Serialize)]
+pub struct ActionsFromYaml(Vec<Action>);
+
+impl ActionsFromYaml {
+    /// Get a reference to the actions from yaml's actions.
+    pub fn actions(&self) -> &[Action] {
+        self.0.as_ref()
+    }
+}
